# Reference :
#	https://stackoverflow.com/a/48475541
#	https://stackoverflow.com/a/32190892
#	https://developers.google.com/youtube/v3/docs/search/list


import requests
from .YoutubeAPI.converter import convert

class Youtube:
	def __init__(self, url:str, api_key:str):
		self.key = api_key
		self.video_url = url
		# Get video ID
		video_id = url.split("/")[-1]
		self.video_id = video_id
		
	def title(self):
		# Doesn't need API key
		url = self.video_url
		res = requests.get(f"https://noembed.com/embed?url={url}")
		return str(res.json()["title"])
		
	def author(self):
		# Doesn't need API key
		url = self.video_url
		res = requests.get(f"https://noembed.com/embed?url={url}")
		# [Author name, Author channel url]
		return [res.json()["author_name"], res.json()["author_url"]]
		
	def thumbnail(self):
		# Doesn't need API key
		url = self.video_url
		res = requests.get(f"https://noembed.com/embed?url={url}")
		return str(res.json()["thumbnail_url"])
		
	def duration(self):
		key = self.key
		video_id = self.video_id
		res = requests.get(f"https://www.googleapis.com/youtube/v3/videos?part=snippet%2CcontentDetails%2Cstatistics&id={video_id}&key={key}")
		json = res.json()
		
		for items in json["items"]:
			duration = items["contentDetails"]["duration"]
			
		duration = {
			"actual": convert(duration, "m"),
			"simple": convert(duration)
			}
			
		return duration
		
	def id(self):
		return str(self.video_id)
			
	def definition(self):
		key = self.key
		video_id = self.video_id
		res = requests.get(f"https://www.googleapis.com/youtube/v3/videos?part=snippet%2CcontentDetails%2Cstatistics&id={video_id}&key={key}")
		json = res.json()
	
		for items in json["items"]:
			definition = items["contentDetails"]["definition"]
		
		definition = str(definition)
		return str.upper(definition)
			
	
	def likes(self):
		key = self.key
		video_id = self.video_id
		res = requests.get(f"https://www.googleapis.com/youtube/v3/videos?part=snippet%2CcontentDetails%2Cstatistics&id={video_id}&key={key}")
		json = res.json()
		
		for items in json["items"]:
			likes = items['statistics']['likeCount']
			
		return int(likes)
			
	def views(self):
		key = self.key
		video_id = self.video_id
		res = requests.get(f"https://www.googleapis.com/youtube/v3/videos?part=snippet%2CcontentDetails%2Cstatistics&id={video_id}&key={key}")
		json = res.json()
		
		for items in json["items"]:
			views = items['statistics']['viewCount']
			
		return int(views)
			
	def comments(self):
		key = self.key
		video_id = self.video_id
		res = requests.get(f"https://www.googleapis.com/youtube/v3/videos?part=snippet%2CcontentDetails%2Cstatistics&id={video_id}&key={key}")
		json = res.json()
		
		for items in json["items"]:
			comments = items['statistics']['commentCount']
				
		return int(comments)
			
	def upload_date(self):
		key = self.key
		video_id = self.video_id
		res = requests.get(f"https://www.googleapis.com/youtube/v3/videos?part=snippet%2CcontentDetails%2Cstatistics&id={video_id}&key={key}")
		json = res.json()
		
		for items in json["items"]:
			publish = items['snippet']['publishedAt']
			
		# Idk how to convert RFC 3339 format, so i tricked with this :)
		publish = str(publish)
		publish = publish.replace("T", " ")
		publish = publish.replace("Z", "")
		return publish
		
	def description(self):
		key = self.key
		video_id = self.video_id
		res = requests.get(f"https://www.googleapis.com/youtube/v3/videos?part=snippet%2CcontentDetails%2Cstatistics&id={video_id}&key={key}")
		json = res.json()
		
		for items in json["items"]:
			description = items["snippet"]["description"]
			
		return description
		
	def search(self, 
		query:str, 
		max_results:int=10,
		type:str="video",
		sort_by:str="relevance",
		video_definition:str="any",
		safesearch_type:str="moderate"
		):
			
		if sort_by == "view":
			sort_by = "viewCount"
		elif sort_by == "video":
			sort_by = "viewCount"
		
		key = self.key
		query = query.replace(" ", "%20")
		
		results = []
		req = requests.get(f"https://youtube.googleapis.com/youtube/v3/search?part=snippet&safeSearch={safesearch_type}&order={sort_by}&q=v{query}&type={type}&videoDefinition={video_definition}&maxResults={max_results}&key={key}")
		parse = req.json()
		
		for items in parse["items"]:
			title = items["snippet"]["title"]
			video_id = items["id"]["videoId"]
			video_link = "https://youtube.com/watch?v="+video_id
			
			res = {
				"title": title,
				"link": video_link
				}
			results.append(res)
			
		return results



from .discord import Discord
from .anonfiles import Anonfiles
from .randomword import Randomword
from .youtube import Youtube

__title__ = "Whitehat"
__version__ = "1.0"
__authors__ = "LyQuid and EterNomm"
__license__ = "Apache License 2.0"
__copyright__ = "Copyright 2022-present EterNomm"



from .DiscordAPI.webhook import webhook
from .DiscordAPI.send_message import send_message

class Discord:
	class webhook:
		def __init__(self, url:str):
			self.wh = webhook(url)
			
		def message(self,
			username:str=None,
			message:str=None,
			avatar_url:str=None
			):
				
			self.wh.message(username, message, avatar_url)
		
		def timestamp(self):
			return self.wh.timestamp()
					
		def fields(self,
			name:str,
			value:str,
			inline=False
			):

			return self.wh.fields(name, value, inline)
				
		def embed(self,
			title:str, 
			description:str=None,
			url:str=None,
			color:int=None,
			timestamp=None,
			image:str=None,
			author_name:str=None,
			author_url:str=None,
			author_icon:str=None,
			fields:list=None,
			thumbnail_url:str=None,
			footer_text:str=None,
			footer_icon:str=None
		        ):
			
			self.wh.embed(
			title,
			description,
			url,
			color,
			timestamp,
			image,
			author_name,
			author_url,
			author_icon,
			fields,
			thumbnail_url,
			footer_text,
			footer_icon
			)
			
		def post(self):
			return self.wh.post()
			
			
			
	class send_message:
		def __init__(self, token:str, channel_id:int, api_version:int=9):
			self.sm = send_message(token, channel_id, api_version)
		
		def tag_user(self, user_id:int):
			return self.sm.tag_user(user_id)
		
		def send(self, message:str):
			return self.sm.send(message)



import requests

class Randomword:
	def __init__(self, number:int):
		if isinstance(number, str) == True:
			raise TypeError('"number" argument cannot string, must integer.')
		base_url = f"https://random-word-api.herokuapp.com/word?number={number}"
		self.response = requests.get(base_url)
		
	def list(self):
		json = self.response.json()
		return '\n'.join(str(x) for x in json)
		
	def json(self):
		return self.response.json()



import requests

class Anonfiles:
	class upload:
		def __init__(self, file:str):
			payload = {
				"file": (file, open(file, "rb"))
			}
		
			self.res = requests.post("https://api.anonfiles.com/upload", payload=payload)
		
		def json(self):
			return self.res.json()
			
		def text(self):
			json = self.res.json()
			
			status = json['status']
			short_url = json['data']['file']['url']['short']
			full_url = json['data']['file']['url']['full']
			size = json['data']['file']['metadata']['size']['readable']
			filename = json['data']['file']['metadata']['name']
			file_id = json['data']['file']['metadata']['id']
			
			return f"Status : {status}\nShort url : {short_url}\nFull url : {full_url}\nSize : {size}\nFilename : {filename}\nFile ID : {file_id}"
			
	
	class info:
		def __init__(self, file_id:str):
			res = requests.get(f"https://api.anonfiles.com/v2/file/{file_id}/info")
			self.res_json = res.json()
			
		def json(self):
			return self.res_json
			
		def text(self):
			status = self.res_json['status']
			short_url = self.res_json['data']['file']['url']['short']
			full_url = self.res_json['data']['file']['url']['full']
			size = self.res_json['data']['file']['metadata']['size']['readable']
			filename = self.res_json['data']['file']['metadata']['name']
			file_id = self.res_json['data']['file']['metadata']['id']
			
			return f"Status : {status}\nShort url : {short_url}\nFull url : {full_url}\nSize : {size}\nFilename : {filename}\nFile ID : {file_id}"



from setuptools import setup, find_packages
import re

with open('README.md') as f:
    long_description = f.read()

version = ''
with open('manyapi/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Version is not set')

def read_requirements():
    with open('requirements.txt', 'r') as req:
        content = req.read()
        requirements = content.split('\n')

    return requirements


setup(
    name='manyapi',
    version=version,
    author='EterNomm',
    author_email='eternommorg@gmail.com',
    description = 'All in one API wrapper.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='GNU General Public License 3.0',
    url='https://github.com/EterNomm/ManyAPI',
    project_urls={
        "Source Code": "https://github.com/EterNomm/ManyAPI",
        "Discord": "https://discord.gg/qpT2AeYZRN",
        "Issue tracker": "https://github.com/EterNomm/ManyAPI/issues"
    },
    packages=find_packages(),
    include_package_data=True,
    install_requires=read_requirements(),
    keywords=["python", "api", "requests", "manyapi", "all in one api"],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Android'
    ]
)
